#!/bin/bash
# $File: lib/function.inc.sh $
# $Author: Marko Schulz <info@tuxnet24.de> $
# $Date: 2018-12-21 07:55:14 +0200 (Fr, 21 Dez 2018) $
# $HeadURL: https://gitbucket.tuxnet24.de/git/tuxnet24/letsencrypt.git $
# $Description: Function library for the letsencrypt script. $
# *******************************************************
# This function display help/usage and exit the program.
# If an argument exists (error message) the program will
# be exit with exit code 1.
#
# @param $1 - error message (optional)
# @return void
#
function usage () {

local msgerr=$*
local ecode=0

if [ -n "${msgerr}" ]; then
    echo -e "\n\e[1;31m\e[1mERROR: ${msgerr}\e[0m\n"
    ecode=1
fi

echo -e "\aUsage $(basename $0) {create|renew|delete|force} <domain>"
echo -e "  create -    create a new wildcard certificate for the defined domain"
echo -e "   renew -    renew a avaialbe wildcard certificate for the defined domain"
echo -e "  delete -    delete a available wildcard certificate for the defined domain"
echo -e "   force -    delete files of wildcard certificate like example-0001"
echo -e "    list -    display all available letsencrypt certificates on this server"
echo -e "  deploy -    deploy the ssl certificates packages to this server"
echo -e "  --help -    display the help/usage screen\n"
exit $ecode

}

# *******************************************************
# This function use the perl select command to create
# a break of ms, and not only for s.
#
# @params $1 - The seconds (default: 1.0, but you can use also 0.5)
# @return void
#
function sleep () {

local ms=${1:-1.0}
perl -e 'select(undef, undef, undef, '$ms');'

}

# *********************************************************
# This function split a string by a string into an arryay.
#
# @param $1 - separator
# @param $2 - string
# @return array
#
function explode() {

local separator=$1; shift
local string=$1
local -a array
local OIFS=$IFS

IFS="${separator}"
read -d '' -r -a array <<< "${string}"
echo -n "${array[@]}"
IFS=$OIFS

}

# *******************************************************
# This function converts time designations
# such as 1d or 12h into seconds.
#
# @param $1 - The time designations
# @return init
#
function get_seconds () {

local item=$1
local digit=$(echo $item | perl -n -e 'print $1," ",$2 if ($_ =~ m/^(\d+)((?:d|h|m|s))$/g);' | awk '{print $1}')
local unit=$(echo $item | perl -n -e 'print $1," ",$2 if ($_ =~ m/^(\d+)((?:d|h|m|s))$/g);' | awk '{print $2}')
local sec
case $unit in
	d) sec=$((${digit}*86400)) ;;
	h) sec=$((${digit}*3600))  ;;
	m) sec=$((${digit}*60))    ;;
	s) sec=${digit}            ;;
esac
echo $sec

}

# *********************************************************
# This function check, if domain is no subdomain. (Right format <domain>.<tld>)
#
# @param $1 - $ACTION
# @param $2 - $DOMAIN
# @return void
#
function check () {

# Check, if $DOMAIN is an vailed domain (not subdomain)
if [ $(echo ${2} | perl -ne 'print ($_ =~ m/^[a-zA-Z0-9-]{1,61}\.[a-zA-Z]{2,}$/ ? 1 : 0);') -ne 1 ]; then
    if [ "${1}" != "force" ]; then
        usage "You have to define a domain name (format: <domain>.<tld>)"
    else
        if [ ! -n "${2}" ]; then
            usage "You have to define a domain name (format: <domain>.<tld>)"
        fi
    fi
fi

}

# *********************************************************
# This function delete the certificate package from htdocs,
# and if $2 is true, then also delete all from /etc/letsencrypt
# of the defined domain.
#
# @param $1 - the domain
# @param $2 - delete all entrys of certpath (optional: true|false)
# @return bool
#
function delete_package () {

local force="${2:-false}"
local ecode=0

if [ $force == "true" ]; then
    # Remove all entrys of the defined domain in the cfg[certpath]
    rm -rf ${cfg[certpath]}/{live,renewal,archive}/{${1},${1}.conf} || ecode=1
fi

if [ $ecode -eq 0 ]; then
    if [ -f ${cfg[deploypath]}/${1}.tar ]; then
        rm ${cfg[deploypath]}/${1}.tar 2>/dev/null
        return $?
    else
        return 0
    fi
else
    return $ecode
fi

}

# *********************************************************
# This function add the post_hook command to the renewal config.
#
# @param $1 - the domain
# @return bool
#
function add_posthook () {

local domain=$1

# Sleep for 1/2 second and go on
sleep 0.5

if [ -f ${cfg[certpath]}/renewal/${domain}.conf ]; then

    local hookstr="${cfg[posthook]} \\\"${domain}\\\""
    hookstr=$(echo $hookstr | sed 's/ /#/g'| sed 's/\//\\\//g')

    local hook_found=$( perl -ne 'BEGIN {$found=0;} $found=1 if($_ =~ m/^post_hook/ ? 1 : 0); END {print $found;}' ${cfg[certpath]}/renewal/${domain}.conf )

    if [ $hook_found -eq 1 ]; then
        perl -p -ne '
            BEGIN {
            $hookstr = "'$hookstr'";
            $hookstr =~ s/#/ /g;
            }
            s/^(post_hook\s+=\s+)\W+(.+?)\W+$/\1\047$hookstr\047\n/g;
        ' ${cfg[certpath]}/renewal/${domain}.conf
    else
        perl -p -ne '
            BEGIN {
            $hookstr = "'$hookstr'";
            $hookstr =~ s/#/ /g;
            }
            s/^(\[renewalparams\])$/\1\npost_hook = \047$hookstr\047/g;
        ' ${cfg[certpath]}/renewal/${domain}.conf
    fi

    return $?
else
    return 0
fi

}

# ***********************************************************
# This function edit a defined line in a defined file, separated by | char.
#
# @param: string $1 - The path to the file
# @param: string $2 - The package name (first item)
# @param: string $3 - The timestamp (second item)
# @return: void
#
function mod () {

local file=$1
local key=$2
local val=$3

if awk -F '|' '{if ($1 == "'$key'") { print $1"|"'$val'} else { print $0 }}' ${file} >${file}.tmp ; then
    mv ${file}.tmp ${file}
fi

}

# ***********************************************************
# This function get the timestamp of a defined packages
#
# @param: string $1 - The path to the file
# @param: string $2 - The package name (first item)
# @return: string
#
function get () {

local file=$1
local key=$2

local tstamp=$(grep ^$key $file 2>/dev/null | cut -d '|' -f2)
[ ! -n "${tstamp}" ] && tstamp=0
echo $tstamp

}

# ***********************************************************
# This function check if a defined package exist in database.
#
# @param: string $1 - The path to the file
# @param: string $2 - The package name (first item)
# @return: bool
#
function isset () {

local file=$1
local key=$2

if grep -q $key $file 2>/dev/null; then
        true
else
        false
fi

}

# ***********************************************************
# This function delete a defined package from database.
#
# @param: string $1 - The path to the file
# @param: string $2 - The path to the goal directory
# @param: string $3 - The package name (first item)
# @return: void
#
function del () {

local file=$1
local dirs=$2
local key=$3

if awk -F '|' '{if ($1 != "'$key'") print $1"|"'$val'}' ${file} >${file}.tmp ; then
    mv ${file}.tmp ${file}
        dir=$(echo $key | sed 's/\.tar//g')
        [ -d "${dirs}/${dir}" ] && rm -r ${dirs}/${dir}
fi

}

# ***********************************************************
# This function add a new package to the database.
#
# @param: string $1 - The path to the file
# @param: string $2 - The package name (first item)
# @param: string $3 - The timestamp (second item)
# @return: void
#
function add () {

local file=$1
local key=$2
local val=$3

echo "${key}|${val}" >>${file}

}

# *************************************************************
# This function write log messages to a defined logfile.
#
# @param string $file - the path to the logfile
# @param string $state - the log state (e.g. INFO|WARN|ERROR)
# @param string $* - the log message
# @return void
#
function logging () {

local file=$1; shift
local state=$1; shift

if [[ "${TERM}" =~ ^xterm.* ]]; then
    echo -e "[$(date +%Y-%m-%d\ %H:%M:%S)] $state $*" | tee -a $file 2>&1
else
    echo -e "[$(date +%Y-%m-%d\ %H:%M:%S)] $state $*" >>$file 2>&1
fi

}

# *********************************************************
# This function create a new letsencrypt wildcard certificate
# with certbot.
#
# @param $1 - the domain
# @param $2 - comma separated list of additional domains
# @return void
#
function create () {

local domain=$1
local adddomain=$2
local ecode=0

# Check domain
check "${ACTION}" "${domain}"

# Clean the list of SubjectAltNames domains.
# Remove $domain from $adddomains.
local additional=$(
    echo -e "${adddomain}" | \
        perl -p -ne '
            BEGIN { $domain="'${domain}'"; }

            s/^${domain}\,//g;
            s/\,${domain}$//g;
            s/^(.+)(\,${domain})(\,.+)$/\1\3/g;

        '
)

# Create an array of the string $additional
declare -a doms=$( explode "," ${additional} )

# The first part of the certbot create command
local cmd="certbot --duplicate certonly --dns-cloudflare --dns-cloudflare-credentials ${cfg[credentialfile]} --post-hook \"${cfg[posthook]} \\\"${domain}\\\"\" -d ${domain} -d *.${domain} "

# If we have additional domains, we add this to the certbot create command
local dom
for dom in ${doms[@]}; do
    cmd="${cmd} -d ${dom} -d *.${dom}"
done

# Run the certbot create command with eval
eval "${cmd}"

# Get the exit code of the certbot command
ecode=$?

# If all fine, we add the post_hook command to the renewal config of the domain
if [ $ecode -eq 0 ]; then
    if add_posthook ${domain} ; then
        return 0
    else
        return 1
    fi
else
    return $ecode
fi

}

# *********************************************************
# This function delete a letsencrypt wildcard certificate
# with certbot.
#
# @param $1 - the domain
# @return void
#
function delete () {

# Check domain
check "${ACTION}" "${1}"

# Run certbot command to delete a wildcard certificate defined by domain
if ${cfg[certbotbin]} delete --cert-name ${1} ; then
    if delete_package ${1} ; then
        return 0
    else
        return 1
    fi
else
    return 1
fi

}

# *********************************************************
# This function delete a letsencrypt wildcard certificate
# with certbot.
#
# @param $1 - the domain
# @return void
#
function force () {

local ecode=0

# Check domain
check "${ACTION}" "${1}"

if [ ! -d "${cfg[certpath]}/live/${1}" -a ! -n "${1}" ]; then
    usage "You have to define a domain name (format: <domain>.<tld>)"
else
    if delete_package ${1} "true"; then
        return 0
    else
        return 1
    fi
fi

}

# *********************************************************
# This function renew a defined letsencrypt wildcard certificate
# with certbot.
#
# @param $1 - the domain
# @return bool
#
function renew () {

# Check domain
check "${ACTION}" "${1}"

if [ ! -d "${cfg[certpath]}/live/${1}" -a ! -n "${1}" ]; then
    usage "You have to define a domain name (format: <domain>.<tld>)"
else
    ${cfg[certbotbin]} -q renew --cert-name ${1}
fi

return $?

}

# *********************************************************
# This function liste all available certificates from cfg[certpath].
#
# @return string
#
function list () {

# Get the current unix timestamp
local now=$( date +%s )

# Get second of expire period for warning
local warning=$( get_seconds ${cfg[expired]} )

# Loop all directory within the certificate directory
local dirs
for dirs in $(find "${cfg[certpath]}/" -mindepth 1 -type d | sed 's/ /@/g'); do

    # Get the original path with spaces
    local path=$(echo ${dirs} | sed 's/@/ /g')

    # Get the domain name from the path
    local domain=$(basename "${path}")

    # Only if a certificate file exists
    if [ -f "${path}/cert.pem" ]; then

        # Get the expire date from the certificate file with openssl
        local enddate=$(openssl x509 -enddate -noout -in "${path}/cert.pem" | awk -F= '{print $NF}')
	# Get unix timestamp of the expire date
        local expire=$(date -d "${enddate}" +%s)
	# Display expire date like 25.12.2023
        enddate=$(date -d "${enddate}" +%d.%m.%Y)

	# Calculate the warnig unix timestamp
        local warn=$[$expire-$warning]

	# Calculate Normal, Warning, Expiered
        if [ $now -ge $warn -a $now -lt $expire ]; then
            echo -e " [\e[1;33m\e[1m${enddate}\e[0m] - ${domain}" # yellow
        elif [ $now -ge $expire ]; then
            echo -e " [\e[1;31m\e[1m${enddate}\e[0m] - ${domain}" # red
        else
            echo -e " [\e[1;32m\e[1m${enddate}\e[0m] - ${domain}" # green
        fi

    fi

done

echo

}

# vim: syntax=bash ts=2 sw=2 sts=2 sr noet
# EOF
